@@ -7,24 +7,66 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
-   [14]={ 0, 0, 0 },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["left"]=0,
@@ -35,32 +77,128 @@ local jfm={
   ["chars"]={ "（", "〔", "［", "｛", "〈", "《", "「", "『", "【", "｟", "〘", "〖", "«", "〝", "‘", "“" },
   ["depth"]=0.5,
   ["glue"]={
-   [0]={ 0, 0, 0 },
-   { 0, 0, 0 },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   [0]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
-   [8]={ 0, 0, 0 },
-   [9]={ 0, 0, 0 },
-   [10]={ 0, 0, 0 },
-   [11]={ 0, 0, 0 },
-   [12]={ 0, 0, 0 },
-   [13]={ 0, 0, 0 },
-   [14]={ 0, 0, 0 },
-   [15]={ 0, 0, 0 },
-   [16]={ 0, 0, 0 },
-   [27]={ 0, 0, 0 },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [8]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [9]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [10]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [11]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [12]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [13]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [15]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [16]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [27]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=0.5,
@@ -75,6 +213,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -82,14 +221,21 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
-   { 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [4]={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -97,15 +243,27 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -113,6 +271,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -120,6 +279,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -127,6 +287,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -134,6 +295,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -141,6 +303,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -148,12 +311,14 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [15]={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -161,6 +326,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -168,6 +334,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -175,6 +342,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -182,6 +350,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -189,6 +358,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -205,68 +375,127 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    {
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [9]={ 0, 0, 0 },
-   [10]={ 0, 0, 0 },
-   [11]={ 0, 0, 0 },
+   [9]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [10]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [11]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [12]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [13]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [15]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [16]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=1,
@@ -281,6 +510,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -288,6 +518,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -295,6 +526,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -302,6 +534,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -309,6 +542,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0.5,
    },
@@ -316,6 +550,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -323,6 +558,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -330,6 +566,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -337,6 +574,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -344,6 +582,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -351,6 +590,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -358,6 +598,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -365,6 +606,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -372,6 +614,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -379,6 +622,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -386,6 +630,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -393,6 +638,7 @@ local jfm={
     0.25,
     0.25,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -1 },
     ["ratio"]=0,
    },
@@ -400,6 +646,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -407,6 +654,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -414,6 +662,7 @@ local jfm={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0,
    },
@@ -431,6 +680,7 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -438,29 +688,48 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
-   { 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [4]={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
    [5]={
     0.75,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0.33333333,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -468,24 +737,28 @@ local jfm={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
    [10]={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
    [11]={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
    [12]={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -493,6 +766,7 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -500,6 +774,7 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -507,6 +782,7 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -514,6 +790,7 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -521,6 +798,7 @@ local jfm={
     0.5,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
     ["ratio"]=0,
    },
@@ -528,18 +806,21 @@ local jfm={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
    [301]={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
    [302]={
     0.5,
     0,
     0,
+    ["kanjiskip_natural"]=1,
     ["ratio"]=0,
    },
   },
@@ -556,6 +837,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -563,14 +845,21 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
-   { 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [4]={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -578,15 +867,27 @@ local jfm={
     0.75,
     0,
     0.75,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=0.33333333,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -594,6 +895,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -601,6 +903,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -608,6 +911,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -615,6 +919,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -622,6 +927,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -629,6 +935,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -636,6 +943,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -643,6 +951,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -650,6 +959,7 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=0,
    },
@@ -657,6 +967,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -664,6 +975,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -671,6 +983,7 @@ local jfm={
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=0,
    },
@@ -687,37 +1000,81 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
    {
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={ 0, 0, 0 },
-   [14]={ 0, 0, 0 },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=1,
@@ -731,35 +1088,74 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -772,35 +1168,74 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -813,35 +1248,74 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -854,36 +1328,80 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
    {
     0.5,
     0,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
-   [14]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -896,41 +1414,81 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
    {
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -943,73 +1501,133 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    {
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [9]={ 0, 0, 0 },
-   [10]={ 0, 0, 0 },
-   [11]={ 0, 0, 0 },
+   [9]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [10]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [11]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [12]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [13]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [15]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [16]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=1,
@@ -1023,24 +1641,66 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
-   [14]={ 0, 0, 0 },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -1053,24 +1713,66 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
-   [14]={ 0, 0, 0 },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -1082,30 +1784,54 @@ local jfm={
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
    [12]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, 0 },
    },
    [13]={
@@ -1114,10 +1840,30 @@ local jfm={
     0,
     ["priority"]={ 0, 0 },
    },
-   [14]={ 0, 0, 0 },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [14]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
  },
@@ -1138,74 +1884,134 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    {
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [9]={ 0, 0, 0 },
-   [10]={ 0, 0, 0 },
-   [11]={ 0, 0, 0 },
+   [9]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [10]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [11]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [12]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [13]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [15]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [16]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=0.25,
@@ -1219,74 +2025,134 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    {
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [9]={ 0, 0, 0 },
-   [10]={ 0, 0, 0 },
-   [11]={ 0, 0, 0 },
+   [9]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [10]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [11]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [12]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [13]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [15]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [16]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=0.5,
@@ -1300,74 +2166,134 @@ local jfm={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    {
     0.5,
     0.25,
     0.5,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, -2 },
     ["ratio"]=1,
    },
-   { 0, 0, 0 },
-   [4]={ 0, 0, 0 },
+   {
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [4]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [5]={
     0.25,
     0,
     0.25,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ 0, -1 },
     ["ratio"]=1,
    },
-   [6]={ 0, 0, 0 },
-   [7]={ 0, 0, 0 },
+   [6]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [7]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [8]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [9]={ 0, 0, 0 },
-   [10]={ 0, 0, 0 },
-   [11]={ 0, 0, 0 },
+   [9]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [10]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [11]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
    [12]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [13]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [14]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [15]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [16]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
    [27]={
     0,
     0.25,
     0,
+    ["kanjiskip_natural"]=1,
     ["priority"]={ -1, 0 },
    },
-   [300]={ 0, 0, 0 },
-   [301]={ 0, 0, 0 },
-   [302]={ 0, 0, 0 },
+   [300]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [301]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
+   [302]={
+    0,
+    0,
+    0,
+    ["kanjiskip_natural"]=1,
+   },
   },
   ["height"]=0.5,
   ["width"]=1,
@@ -1376,8 +2302,8 @@ local jfm={
  ["kanjiskip"]={ 0, 0.25, 0 },
  ["version"]=3,
  ["xkanjiskip"]={ 0.25, 0.25, 0.125 },
- ["zh"]=1,
- ["zw"]=1,
+ ["zh"]=1.0,
+ ["zw"]=1.0,
 }
 local function add_space(before,after,glueorkern,space,ratio)
 	if jfm[before][glueorkern] == nil then jfm[before][glueorkern] = {} end
