@@ -6,7 +6,6 @@
 -- kanjiskip:    0pt plus .25zw minus 0pt
 -- xkanjiskip: .25zw plus .25zw (or .0833zw) minus .125zw
 
-
 local t = {
    version = 3,
    dir = 'tate',
@@ -26,7 +25,9 @@ local t = {
 	 [107] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [207] = { 0, 0, 0, kanjiskip_shrink=1 },
          [8] = { 0, 0, 0, kanjiskip_shrink=1 },
-      }
+      },
+      kern = { [307] = 0 },	 
+      round_threshold = 0.01,
    },
 
    [1] = { -- 開き括弧類
@@ -135,7 +136,7 @@ local t = {
       }
    },
 
-   [5] = { -- -- 二分（二重）ダッシュ
+   [105] = { -- -- 二分（二重）ダッシュ
       chars = { '゠', '–' },
       align = 'middle', left = 0.0, down = 0.0,
       width = 0.50, height = 0.50, depth = 0.50, italic=0.0,
@@ -221,10 +222,23 @@ local t = {
 	 [3] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
 	 [4] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [6] = { 0, 0, 0, kanjiskip_shrink=1 },
-	 [107] = { 0, 0, 0, kanjiskip_shrink=1 },
+	 [207] = { 0, 0, 0, kanjiskip_shrink=1 },
          [8] = { 0, 0, 0, kanjiskip_shrink=1 },
       }
    },
+   [307] = { -- 合成用濁点・半濁点
+      chars = { 0x3099, 0x309A },
+      align = 'right', left = 0.0, down = 0.0,
+      width = 0, height = 0.5, depth = 0.5, italic=0.0,
+      glue = {
+	 [1] = { 0.5 , 0.0, 0.5, ratio=1, kanjiskip_stretch=1 },
+	 [2] = { 0, 0, 0, kanjiskip_shrink=1 },
+	 [3] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
+	 [4] = { 0, 0, 0, kanjiskip_shrink=1 },
+	 [6] = { 0, 0, 0, kanjiskip_shrink=1 },
+    [8] = { 0, 0, 0, kanjiskip_shrink=1 },
+      }
+   },
 
    [8] = { -- 罫線類．
       chars = {
@@ -260,7 +274,10 @@ local t = {
    },
 
    [99] = { -- box末尾
-      chars = {'boxbdd', 'parbdd', 'glue'},
+      chars = {'boxbdd', 'glue'},
+   },
+   [199] = { -- box末尾
+      chars = {'parbdd'},
    },
 
 }
@@ -283,7 +300,6 @@ end
 t[200]=table.fastcopy(t[0])
 t[200].chars={'〱', '〲'}
 t[200].width = 2.0
-t[200].left=-0.5
 for i,v in pairs(t) do
   if type(v)=='table' then
     if v.glue and v.glue[0] then v.glue[200] = v.glue[0] end
@@ -291,4 +307,9 @@ for i,v in pairs(t) do
   end
 end
 
+local jf = luatexja.jfont.jfm_feature
+if jf and jf.beginpar_middledot_zw==true then
+    t[199].kern = { [3] = 0.25 }
+end
+
 luatexja.jfont.define_jfm(t)
