@@ -319,7 +319,7 @@ end
 local create_dir_whatsit_vbox
 do
    local wh = {}
-   local id_glue, sid_parskip = node.id 'glue', 3
+   local id_glue = node.id 'glue'
    create_dir_whatsit_vbox = function (hd, gc)
       ltjs.list_dir = get_dir_count()
       -- remove dir whatsit
@@ -331,8 +331,8 @@ do
       if hd==wh[1] then
          ltjs.list_dir =has_attr(hd,attr_dir)
          local x = node_next(hd)
-         if getid(x)==id_glue and getsubtype(x)==sid_parskip then
-            node_remove(hd,x); node_free(x)
+         while x and getid(x)==id_glue and getsubtype(x)==3 do
+            node_remove(hd,x); node_free(x); x = node_next(hd)
          end
       end
       for i=1,#wh do
@@ -345,10 +345,10 @@ do
          end
          return hd
       else
-         local n =node_next(hd)
+         local n = node_next(hd)
          if gc=='vtop' then
             local w = create_dir_whatsit(hd, gc, ltjs.list_dir)
-            -- move  dir whatsit after hd
+            -- move dir whatsit after hd
             setfield(hd, 'next', w); setfield(w, 'next', n)
             return hd
          else
@@ -1113,21 +1113,18 @@ do
          end
       end
    end
-   local getbox, shipout = tex.getbox, tex.shipout
-   local setbox, copy = node.direct.setbox, node.direct.copy
-   local afbox
-   function luatexja.direction.shipout()
+   local copy = node.direct.copy
+   function luatexja.direction.shipout_lthook (head)
       start_time_measure 'box_primitive_hook'
-      if (not afbox) and luatexja.afbox_number  then afbox = luatexja.afbox_number end
-      local a = to_direct(getbox 'ltj@afbox')
+      local a = to_direct(head)
       local a_dir = get_box_dir(a, dir_yoko)
       if a_dir~=dir_yoko then
          local b = create_dir_node(a, a_dir, dir_yoko, false)
          setfield(b, 'head', a); a = b
       end
       setfield(shipout_temp, 'head', a); finalize_inner(shipout_temp)
-      setbox(afbox, copy(getlist(shipout_temp))); setfield(shipout_temp, 'head',nil)
-      shipout(afbox)
+      a = copy(getlist(shipout_temp)); setfield(shipout_temp, 'head',nil)
       stop_time_measure 'box_primitive_hook'
+      return to_node(a)
    end
 end
