@@ -3,7 +3,7 @@
 --
 luatexbase.provides_module({
   name = 'luatexja.jfmglue',
-  date = '2020-12-22',
+  date = '2022-01-02',
   description = 'Insertion process of JFM glues, [x]kanjiskip and others',
 })
 luatexja.jfmglue = luatexja.jfmglue or {}
@@ -45,11 +45,12 @@ local node_next = node.direct.getnext
 local ltjd_make_dir_whatsit = ltjd.make_dir_whatsit
 local ltjf_font_metric_table = ltjf.font_metric_table
 local ltjf_find_char_class = ltjf.find_char_class
-local node_new = node.direct.new
+local node_new = luatexja.dnode_new
 local node_copy = node.direct.copy
 local node_tail = node.direct.tail
 local node_free = node.direct.free
 local node_remove = node.direct.remove
+local node_inherit_attr = luatexja.node_inherit_attr
 
 local id_glyph = node.id 'glyph'
 local id_hlist = node.id 'hlist'
@@ -347,8 +348,7 @@ function calc_np_aux_glyph_common(lp, acc_flag)
       local lx=lp
       while lx do
          local lai = get_attr_icflag(lx)
-         if lx==last or  lai>=PACKED then
-            lp=lx; break
+         if lx==last or  lai>=PACKED then break
          else
             local lid = getid(lx)
             if lid==id_glyph and not if_lang_ja(lx) then
@@ -368,22 +368,23 @@ function calc_np_aux_glyph_common(lp, acc_flag)
                   else -- アクセントは上下にシフトされている
                      setfield(lx, 'shift', getfield(lx, 'shift') + (has_attr(lx,attr_ablshift) or 0))
                   end
-                  lx = node_next(node_next(lx))
+                  set_attr(lx, attr_icflag, PROCESSED)
+                  lx = node_next(lx); set_attr(lx, attr_icflag, PROCESSED)
+                  lx = node_next(lx); set_attr(lx, attr_icflag, PROCESSED)
                elseif ls==0  then
                   Np.last = lx; lx = node_next(lx)
                elseif (ls==3) or (lai==ITALIC) then
                   Np.last = lx; set_attr(lx, attr_icflag, IC_PROCESSED); lx = node_next(lx)
-               else
-                  lp=lx; break
+               else break
                end
-            else
-               lp=lx; break
+            else break
             end
          end
       end
+      lp=lx
       local r
       if adj_depth>node_depth then
-            r = node_new(id_rule,3)
+            r = node_new(id_rule,3,first_glyph)
             setfield(r, 'width', 0); setfield(r, 'height', 0)
             setfield(r, 'depth',adj_depth); setfield(r, 'dir', tex_dir)
             set_attr(r, attr_icflag, PROCESSED)
@@ -689,7 +690,7 @@ local function handle_penalty_normal(post, pre, g)
    local a = (pre or 0) + (post or 0)
    if #Bp == 0 then
       if (a~=0 and not(g and getid(g)==id_kern)) then
-         local p = node_new(id_penalty)
+         local p = node_new(id_penalty, nil, Nq.nuc, Np.nuc)
          if a<-10000 then a = -10000 elseif a>10000 then a = 10000 end
          setfield(p, 'penalty', a); head = insert_before(head, Np.first, p)
          Bp[1]=p; set_attr(p, attr_icflag, KINSOKU)
@@ -703,7 +704,7 @@ local function handle_penalty_always(post, pre, g)
    local a = (pre or 0) + (post or 0)
    if #Bp == 0 then
       if not (g and getid(g)==id_glue) or a~=0 then
-         local p = node_new(id_penalty)
+         local p = node_new(id_penalty, nil, Nq.nuc, Np.nuc)
          if a<-10000 then a = -10000 elseif a>10000 then a = 10000 end
          setfield(p, 'penalty', a); head = insert_before(head, Np.first, p)
          Bp[1]=p; set_attr(p, attr_icflag, KINSOKU)
@@ -716,7 +717,7 @@ local function handle_penalty_suppress(post, pre, g)
    luatexbase.call_callback('luatexja.adjust_jfmglue', head, Nq, Np, Bp)
    if #Bp == 0 then
       if g and getid(g)==id_glue then
-         local p = node_new(id_penalty)
+         local p = node_new(id_penalty, nil, Nq.nuc, Np.nuc)
          setfield(p, 'penalty', 10000); head = insert_before(head, Np.first, p)
          Bp[1]=p; set_attr(p, attr_icflag, KINSOKU)
       end
@@ -730,7 +731,7 @@ local function handle_penalty_jwp()
    local a = table_current_stack[luatexja.stack_table_index.JWP]
    if #widow_Bp == 0 then
       if a~=0 then
-         local p = node_new(id_penalty)
+         local p = node_new(id_penalty, widow_Np.nuc)
          if a<-10000 then a = -10000 elseif a>10000 then a = 10000 end
          setfield(p, 'penalty', a); head = insert_before(head, widow_Np.first, p)
          widow_Bp[1]=p; set_attr(p, attr_icflag, KINSOKU)
@@ -759,8 +760,7 @@ end
 -- Nq.last (kern w) .... (glue/kern g) Np.first
 local function real_insert(g)
    if g then
-      head  = insert_before(head, Np.first, g)
-      Np.first = g
+      head, Np.first = insert_before(head, Np.first, node_inherit_attr(g, Nq.nuc, Np.nuc))
       local ngk = Np.gk
       if not ngk then Np.gk = g
       elseif type(ngk)=="table" then ngk[#ngk+1]=g
@@ -975,24 +975,20 @@ end
 -- NA, NB: alchar or math
 local function get_NA_skip()
    local pm = Np.met
-   local g, _, kn, kp, kh = new_jfm_glue(
-      pm.char_type,
-      fast_find_char_class(
-        (Nq.id == id_math and -1 or (Nq.xspc>=2 and 'alchar' or 'nox_alchar')), pm), 
-      Np.class)
-   local k = ((Nq.xspc>=2) and (Np.xspc%2==1) and combine_spc 'auto_xspc')
-       and get_xkanjiskip_low(false, pm, kn, kp, kh)
+   local qclass = fast_find_char_class(
+      (Nq.id == id_math and -1 or (Nq.xspc>=2 and 'alchar' or 'nox_alchar')), pm)
+   local g, _, kn, kp, kh = new_jfm_glue(pm.char_type, qclass, Np.class)
+   local k = g and (Nq.xspc>=2) and (Np.xspc%2==1) and combine_spc 'auto_xspc'
+      and get_kanjiskip_low(true, pm, kn, kp, kh)
    return g, k
 end
 local function get_NB_skip()
    local qm = Nq.met
-   local g, _, kn, kp, kh = new_jfm_glue(
-      qm.char_type, Nq.class,
-      fast_find_char_class(
-        (Np.id == id_math and -1 or (Np.xspc%2==1 and 'alchar' or 'nox_alchar')), qm)
-    )
-   local k = ((Nq.xspc>=2) and (Np.xspc%2==1) and combine_spc 'auto_xspc')
-         and get_xkanjiskip_low(false, qm, kn, kp, kh)
+   local pclass = fast_find_char_class(
+      (Np.id == id_math and -1 or (Np.xspc%2==1 and 'alchar' or 'nox_alchar')), qm)
+   local g, _, kn, kp, kh = new_jfm_glue(qm.char_type, Nq.class, pclass)
+   local k = g and (Nq.xspc>=2) and (Np.xspc%2==1) and combine_spc 'auto_xspc'
+      and get_kanjiskip_low(true, qm, kn, kp, kh)
    return g, k
 end
 
@@ -1037,8 +1033,7 @@ local function handle_np_jachar(mode)
       if not g then g = get_kanjiskip() end
       handle_penalty_normal(0, Np.pre, g); real_insert(g); real_insert(k)
    elseif Nq.pre then
-      local g, k
-      if non_ihb_flag then g, k = get_NA_skip() end -- N_A->X
+      local g, k; if non_ihb_flag then g, k = get_NA_skip() end -- N_A->X
       if not g then g = get_xkanjiskip(Np) end
       handle_penalty_normal((qid==id_hlist and 0 or Nq.post), Np.pre, g); 
       real_insert(g); real_insert(k)
@@ -1059,8 +1054,10 @@ end
 -- jachar .. (anything)
 local function handle_nq_jachar()
     if Np.pre then
-      local g = non_ihb_flag and get_NB_skip() or get_xkanjiskip(Nq) -- N_B->X
-      handle_penalty_normal(Nq.post, (Np.id==id_hlist and 0 or Np.pre), g); real_insert(g)
+      local g, k; if non_ihb_flag then g, k =  get_NB_skip()end -- N_B->X
+      if not g then g = get_xkanjiskip(Nq) end
+      handle_penalty_normal(Nq.post, (Np.id==id_hlist and 0 or Np.pre), g);
+      real_insert(g); real_insert(k)
    else
       local g =non_ihb_flag and  (get_OB_skip()) -- O_B
       if Np.id==id_glue then handle_penalty_normal(Nq.post, 0, g)
@@ -1151,7 +1148,7 @@ local function handle_list_head(par_indented)
          if g then
             set_attr(g, attr_icflag, BOXBDD)
             if getid(g)==id_glue and #Bp==0 then
-               local h = node_new(id_penalty)
+               local h = node_new(id_penalty, nil, Np.nuc)
                setfield(h, 'penalty', 10000); set_attr(h, attr_icflag, BOXBDD)
             end
             head = insert_before(head, Np.first, g)
@@ -1424,31 +1421,27 @@ do
            return lx
        end
    end
+   local function special_jaglue_after_inner(lx, lxi, lxi_jfm, kn, bk)
+       local w, st, sh, sto, sho = getglue(kn)
+       if w~=1073741823 then
+           setglue(lx, w, st, sh, sto, sho); set_attr(lx, attr_icflag, lxi)
+       else
+           local m = ltjf_font_metric_table[has_attr(lx, attr_tablshift)]
+           setglue(lx, bk[1], bk[2], bk[3], 0, 0)
+           set_attr(lx, attr_icflag, lxi_jfm)
+       end
+   end
    local function special_jaglue_after(lx)
        if get_attr_icflag(lx)==SPECIAL_JAGLUE then
            lxi=has_attr(lx, attr_yablshift)
            if lxi>=PROCESSED_BEGIN_FLAG then
                lxi = lxi%PROCESSED_BEGIN_FLAG
                if lxi == KANJI_SKIP then
-                   local w, st, sh, sto, sho = getglue(kanji_skip)
-                   if w~=1073741823 then
-                       setglue(lx, w, st, sh, sto, sho); set_attr(lx, attr_icflag, KANJI_SKIP)
-                   else
-                       local m = ltjf_font_metric_table[has_attr(lx, attr_tablshift)]
-                       local bk = m.kanjiskip or null_skip_table
-                       setglue(lx, bk[1], bk[2], bk[3], 0, 0)
-                       set_attr(lx, attr_icflag, KANJI_SKIP_JFM)
-                   end
-               elseif lxi == XKANJI_SKIP then
-                   local w, st, sh, sto, sho = getglue(xkanji_skip)
-                   if w~=1073741823 then
-                       setglue(lx, w, st, sh, sto, sho); set_attr(lx, attr_icflag, XKANJI_SKIP)
-                   else
-                       local m = ltjf_font_metric_table[has_attr(lx, attr_tablshift)]
-                       local bk = m.xkanjiskip or null_skip_table
-                       setglue(lx, bk[1], bk[2], bk[3], 0, 0)
-                       set_attr(lx, attr_icflag, XKANJI_SKIP_JFM)
-                   end
+                   special_jaglue_after_inner(lx, lxi, KANJI_SKIP_JFM, kanji_skip, 
+                     ltjf_font_metric_table[has_attr(lx, attr_tablshift)].kanjiskip or null_skip_table)
+               else --  lxi == XKANJI_SKIP
+                   special_jaglue_after_inner(lx, lxi, XKANJI_SKIP_JFM, xkanji_skip, 
+                     ltjf_font_metric_table[has_attr(lx, attr_tablshift)].xkanjiskip or null_skip_table)
                end
            else
                set_attr(lx, attr_icflag, lxi)
