@@ -3,7 +3,7 @@
 --
 luatexbase.provides_module({
   name = 'luatexja.ruby',
-  date = '2020-10-30',
+  date = '2021-05-04',
   description = 'Ruby annotation',
 })
 luatexja.ruby = {}
@@ -312,7 +312,7 @@ local function enlarge_parent(r, p, tmp_tbl, no_begin, no_end)
    local intmode = floor(tmp_tbl.mode/4)%4
    if no_begin then mapre  = mapre + tmp_tbl.before_jfmgk end
    if no_end   then mapost = mapost + tmp_tbl.after_jfmgk end
-   if (tmp_tbl.mode%4 >=2) and (tmp.pre<0) and (tmp.post<0) then
+   if (tmp_tbl.mode%4 >=2) and (tmp_tbl.pre<0) and (tmp_tbl.post<0) then
        mapre = min(mapre,mapost); mapost = mapre
    end
    if intmode == 0 then --  とりあえず組んでから決める
@@ -544,12 +544,13 @@ local function pre_high(ahead)
              local op = (atr>0) and (old_break_info[atr] or post_intrusion_backup) or 0
              max_allow_pre = max(0, -max_allow_pre - op)
          end
-         if rst.exclude_pre_from_prev_ruby  and ((atr>0) and (old_break_info[-atr]>0) or post_jfmgk_backup) then
+         if rst.exclude_pre_from_prev_ruby  and atr>0 and old_break_info[-atr]
+           and (old_break_info[-atr]>0 or post_jfmgk_backup) then
             -- 「直前のルビが JFM グルーに進入→現在のルビの前文字進入はなし」という状況
             max_allow_pre = 0; rst.exclude_pre_from_prev_ruby=false
          end
          if rst.exclude_pre_jfmgk_from_prev_ruby
-            and (atr>0) and ((old_break_info[atr]  or post_intrusion_backup) > 0) then
+            and atr>0 and ((old_break_info[atr]  or post_intrusion_backup) > 0) then
             -- 「直前のルビが文字に進入→現在のルビの和文処理グルーへの進入はなし」という状況
             rst.before_jfmgk = 0
          end
