@@ -1,6 +1,6 @@
-# $Id: TLCrypto.pm 55178 2020-05-17 16:46:24Z karl $
+# $Id: TLCrypto.pm 59224 2021-05-16 16:50:31Z karl $
 # TeXLive::TLCrypto.pm - handle checksums and signatures.
-# Copyright 2016-2020 Norbert Preining
+# Copyright 2016-2021 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
@@ -12,7 +12,7 @@ use TeXLive::TLConfig;
 use TeXLive::TLUtils qw(debug ddebug win32 which platform
                         conv_to_w32_path tlwarn tldie);
 
-my $svnrev = '$Revision: 55178 $';
+my $svnrev = '$Revision: 59224 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -20,7 +20,7 @@ sub module_revision { return $_modulerevision; }
 
 =head1 NAME
 
-C<TeXLive::TLCrypto> -- checksums and cryptographic signatures
+C<TeXLive::TLCrypto> -- TeX Live checksums and cryptographic signatures
 
 =head1 SYNOPSIS
 
@@ -311,7 +311,7 @@ sub verify_checksum {
   my ($file, $checksum_url) = @_;
   # don't do anything if we cannot determine a checksum method
   # return -2 which is as much as missing signature
-  return($VS_UNSIGNED) if (!$::checksum_method);
+  return($VS_UNSIGNED, "no checksum method found") if (!$::checksum_method);
   my $checksum_file
     = TeXLive::TLUtils::download_to_temp_or_file($checksum_url);
 
@@ -359,7 +359,7 @@ sub verify_checksum {
   # we are still here, so checksum also succeeded
   debug("checksum of local copy identical with remote hash\n");
 
-  return(0);
+  return($VS_VERIFIED);
 }
 
 # emacs-page
@@ -655,7 +655,7 @@ Also the standard modules L<Digest::MD5> and L<Digest::SHA>.
 =head1 AUTHORS AND COPYRIGHT
 
 This script and its documentation were written for the TeX Live
-distribution (L<http://tug.org/texlive>) and both are licensed under the
+distribution (L<https://tug.org/texlive>) and both are licensed under the
 GNU General Public License Version 2 or later.
 
 =cut
