@@ -1,12 +1,12 @@
-# $Id: TLPaper.pm 44232 2017-05-06 23:06:56Z karl $
+# $Id: TLPaper.pm 59224 2021-05-16 16:50:31Z karl $
 # TeXLive::TLPaper.pm - query/modify paper sizes for our various programs
-# Copyright 2008-2017 Norbert Preining
+# Copyright 2008-2021 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLPaper;
 
-my $svnrev = '$Revision: 44232 $';
+my $svnrev = '$Revision: 59224 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -28,11 +28,13 @@ BEGIN {
   @EXPORT = @EXPORT_OK;
 }
 
+my $prg = ($::prg ? $::prg : TeXLive::TLUtils::basename($0));
+
 =pod
 
 =head1 NAME
 
-C<TeXLive::TLPaper> -- TeX Live paper size setting module
+C<TeXLive::TLPaper> -- TeX Live paper size module
 
 =head1 SYNOPSIS
 
@@ -60,7 +62,7 @@ our %paper = (
   },
   "pdftex"   => {
     sub => \&paper_pdftex,
-    default_component => "tex/generic/config",
+    default_component => "tex/generic/tex-ini-files",
     default_file      => "pdftexconfig.tex",
     pkg => "pdftex",
   },
@@ -232,7 +234,7 @@ sub do_paper {
     my $sub = $paper{$prog}{'sub'};
     return(&$sub($texmfsysconfig, @args));
   } else {
-    tlwarn("$0: unknown paper program $prog ($texmfsysconfig,@args)\n");
+    tlwarn("$prg: unknown paper program $prog ($texmfsysconfig,@args)\n");
     return($F_ERROR);
   }
   return ($F_OK); # not reached
@@ -274,7 +276,7 @@ sub find_paper_file {
     }
   }
 
-  debug("$0: found no paper file for $progname (from $cmd)\n") if ! $ret;
+  debug("$prg: found no paper file for $progname (from $cmd)\n") if ! $ret;
   return $ret;
 }
 
@@ -341,7 +343,7 @@ sub paper_pdftex {
 
   return($F_ERROR) unless $inp; 
 
-  open(FOO, "<$inp") || die "$0: open($inp) failed: $!";
+  open(FOO, "<$inp") || die "$prg: open($inp) failed: $!";
   my @lines = <FOO>;
   close(FOO);
 
@@ -396,6 +398,17 @@ sub paper_pdftex {
       for my $p (keys %pdftex_papersize) {
         info("$p\n") unless ($p eq $currentpaper);
       }
+    } elsif ($newpaper eq "--json") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (keys %pdftex_papersize) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      my %foo;
+      $foo{'program'} = "pdftex";
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@ret;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       my @ret = ();
       push @ret, "$currentpaper";
@@ -444,23 +457,23 @@ sub paper_pdftex {
             $lines[$#lines] = $addlines;
           }
         }
-        info("$0: setting paper size for pdftex to $newpaper.\n");
+        info("$prg: setting paper size for pdftex to $newpaper: $outfile\n");
         mkdirhier(dirname($outfile));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outfile);
         if (!open(TMP, ">$outfile")) {
-          tlwarn("$0: Cannot write to $outfile: $!\n");
+          tlwarn("$prg: Cannot write to $outfile: $!\n");
           tlwarn("Not setting paper size for pdftex.\n");
           return($F_ERROR);
         }
         for (@lines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outfile) failed: $!";
+        close(TMP) || warn "$prg: close(>$outfile) failed: $!";
         TeXLive::TLUtils::announce_execute_actions("regenerate-formats");
         # TODO should we return the value of announce_execute action?
         return($F_OK);
       } else {
-        tlwarn("$0: Not a valid paper size for pdftex: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for pdftex: $newpaper\n");
         return($F_WARNING);
       }
     }
@@ -509,7 +522,7 @@ sub paper_dvips {
 
   return($F_ERROR) unless $inp; 
   
-  open(FOO, "<$inp") || die "$0: open($inp) failed: $!";
+  open(FOO, "<$inp") || die "$prg: open($inp) failed: $!";
   my @lines = <FOO>;
   close(FOO);
 
@@ -542,6 +555,12 @@ sub paper_dvips {
       for my $p (@papersizes) {
         info("$p\n"); # first is already the selected one
       }
+    } elsif ($newpaper eq "--json") {
+      my %foo;
+      $foo{'program'} = "dvips";
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@papersizes;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       return(\@papersizes);
     } else {
@@ -570,20 +589,20 @@ sub paper_dvips {
           }
           push @newlines, $lines[$idx];
         }
-        info("$0: setting paper size for dvips to $newpaper.\n");
+        info("$prg: setting paper size for dvips to $newpaper: $outfile\n");
         mkdirhier(dirname($outfile));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outfile);
         if (!open(TMP, ">$outfile")) {
-          tlwarn("$0: Cannot write to $outfile: $!\n");
+          tlwarn("$prg: Cannot write to $outfile: $!\n");
           tlwarn("Not setting paper size for dvips.\n");
           return ($F_ERROR);
         }
         for (@newlines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outfile) failed: $!";
+        close(TMP) || warn "$prg: close(>$outfile) failed: $!";
       } else {
-        tlwarn("$0: Not a valid paper size for dvips: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for dvips: $newpaper\n");
         return($F_WARNING);
       }
     }
@@ -667,7 +686,7 @@ sub paper_context {
 
   return($F_ERROR) unless $inp; 
 
-  open(FOO, "<$inp") || die "$0: open($inp) failed: $!";
+  open(FOO, "<$inp") || die "$prg: open($inp) failed: $!";
   my @lines = <FOO>;
   close(FOO);
 
@@ -703,6 +722,17 @@ sub paper_context {
       for my $p (keys %pdftex_papersize) {
         info("$p\n") unless ($p eq $currentpaper);
       }
+    } elsif ($newpaper eq "--json") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (keys %pdftex_papersize) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      my %foo;
+      $foo{'program'} = "context";
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@ret;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       my @ret = ();
       push @ret, "$currentpaper";
@@ -734,23 +764,23 @@ sub paper_context {
             $lines[$#lines] = $addlines;
           }
         }
-        info("$0: setting paper size for context to $newpaper.\n");
+        info("$prg: setting paper size for context to $newpaper: $outfile\n");
         mkdirhier(dirname($outfile));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outfile);
         if (!open(TMP, ">$outfile")) {
-          tlwarn("$0: Cannot write to $outfile: $!\n");
+          tlwarn("$prg: Cannot write to $outfile: $!\n");
           tlwarn("Not setting paper size for context.\n");
           return($F_ERROR);
         }
         for (@lines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outfile) failed: $!";
+        close(TMP) || warn "$prg: close(>$outfile) failed: $!";
         TeXLive::TLUtils::announce_execute_actions("regenerate-formats");
         # TODO should we return the value of announce_execute action?
         return($F_OK);
       } else {
-        tlwarn("$0: Not a valid paper size for context: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for context: $newpaper\n");
         return($F_WARNING);
       }
     }
@@ -862,6 +892,17 @@ sub paper_do_simple {
       for my $p (@$paplist) {
         info("$p\n") unless ($p eq $currentpaper);
       }
+    } elsif ($newpaper eq "--json") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (@$paplist) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      my %foo;
+      $foo{'program'} = $prog;
+      $foo{'file'} = $inp;
+      $foo{'options'} = \@ret;
+      return \%foo;
     } elsif ($newpaper eq "--returnlist") {
       my @ret = ();
       push @ret, $currentpaper;
@@ -892,23 +933,23 @@ sub paper_do_simple {
         if (!$foundcfg) {
           push @newlines, &$bl($defaultline, $newpaper);
         }
-        info("$0: setting paper size for $prog to $newpaper.\n");
+        info("$prg: setting paper size for $prog to $newpaper: $outp\n");
         mkdirhier(dirname($outp));
         # if we create the outfile we have to call mktexlsr
         TeXLive::TLUtils::announce_execute_actions("files-changed")
           unless (-r $outp);
         if (!open(TMP, ">$outp")) {
-          tlwarn("$0: Cannot write to $outp: $!\n");
+          tlwarn("$prg: Cannot write to $outp: $!\n");
           tlwarn("Not setting paper size for $prog.\n");
           return ($F_ERROR);
         }
         for (@newlines) { print TMP; }
-        close(TMP) || warn "$0: close(>$outp) failed: $!";
+        close(TMP) || warn "$prg: close(>$outp) failed: $!";
         TeXLive::TLUtils::announce_execute_actions("regenerate-formats")
           if ($prog eq "context");
         return($F_OK);
       } else {
-        tlwarn("$0: Not a valid paper size for $prog: $newpaper\n");
+        tlwarn("$prg: Not a valid paper size for $prog: $newpaper\n");
         return($F_WARNING);
       }
     }
